/** -------------------------------------------------- 

 * Hochschule f�r Technik Stuttgart 

 * Fachbereich Vermessung , Informatik und Mathematik 

 * Schellingstr . 24 

 * D - 70174 Stuttgart 

 * 

 * Projekt CityDoktor 

 * 

 * Copyright (c) 2011 HFT Stuttgart. All rights reserved. 

 * HFT Stuttgart and its licensors retain all intellectual property and 

 * proprietary rights in and to this software and related documentation. 

 * Any use, reproduction, disclosure, or distribution of this software 

 * and related documentation without an express license agreement from 

 * HFT Stuttgart is strictly prohibited. 

 * 

 * Please refer to the applicable HFT Stuttgart end user license agreement (EULA) 

 * associated with this source code for terms and conditions that govern 

 * your use of this HFT Stuttgart software. 

 * 

 * @author Coors

 * @version 1.0

 * created: 26.3.2013

 */

package de.hft.stuttgart.graphtheory;



import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Deque;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;







public class DirectedGraph {
    
    
    
    static int numberPreOrd;
    
    
    
    public DirectedGraph() {
	
	nodes = new HashMap<Integer, DirectedGraphNode>();
	
    }
    
    
    
    static HashMap<Integer, DirectedGraphNode> nodes;
    
    public static HashMap<Integer, DirectedGraphNode> getNodes() {
	return nodes;
    }
    
    
    
    List<List<Integer>> cycles = new ArrayList<List<Integer>>();
    
    
    
    public void add(int start, int end, boolean directed) {
	
	int v0 = start;
	
	int v1 = end;
	
	
	
	////System.out.println("Startpunkt= " + v0 + "Endpunkt= " + v1);
	
	
	
	if (!nodes.containsKey(v0)) {
	    
	    nodes.put(v0, new DirectedGraphNode(v0));
	    
	}
	
	if (!nodes.containsKey(v1)) {
	    
	    nodes.put(v1, new DirectedGraphNode(v1));
	    
	}
	
	
	
	// add edge v0, v1
	
	DirectedGraphNode n0 = nodes.get(v0);
	
	n0.addAdjacentNode(nodes.get(v1));
	
	if (!directed) {
	    
	    DirectedGraphNode n1 = nodes.get(v1);
	    
	    n1.addAdjacentNode(nodes.get(v0));
	    
	}
	
    }
    
    
    
    /*

     * this method is used for testing only. A start node for the cycle search will be given.

     * 

     * @param start - start node for cycle search

     * @param directed - true if directed graph, false if "pseudo undirected" graph

     */
    
    public void findCycles(int start, boolean directed) {
	
	
	
	
	
	numberPreOrd = 1;
	
	findCycles(nodes.get(start), new ArrayDeque<DirectedGraphNode>(nodes.size()),
		
		directed);
	
	//
	
	// nodes durchsuchen
	
	int count = 0;
	
	Iterator<DirectedGraphNode> enum_nodes = nodes.values().iterator();
	
	while (enum_nodes.hasNext()) {
	    
	    DirectedGraphNode w = enum_nodes.next();
	    
	    if (w.getStatus() != DirectedGraphNode.DONE) {
		
		////System.out.println("cycle search= " + count);
		
		findCycles(w, new ArrayDeque<DirectedGraphNode>(nodes.size()), directed);
		
		
		
		count++;
		
	    }
	    
	}
	
	////System.out.println("Anzahl disconnected segments= " + count);
	
	// if nodes are still not marked, graph is not connected
	
	
	
    }
    
    /*

     * search cycles in graph

     * 

     * @param directed - true if directed graph, false if "pseudo undirected" graph

     */
    
    public void findCycles(boolean directed) {
	
	
	
	// nodes durchsuchen
	
	
	
	Iterator<DirectedGraphNode> enum_nodes = nodes.values().iterator();
	
	while (enum_nodes.hasNext()) {
	    
	    DirectedGraphNode w = enum_nodes.next();
	    
	    if (w.getStatus() != DirectedGraphNode.DONE) {
		
		findCycles(w, new ArrayDeque<DirectedGraphNode>(nodes.size()), directed);
		
	    }
	    
	}
	
	
	
    }
    
    
    
    /*

     * recursive cycle search using depth search first

     * 

     */
    
    private void findCycles(DirectedGraphNode v, Deque<DirectedGraphNode> path, boolean directed) {
	
	if (v.getStatus() == DirectedGraphNode.IN_PROGRESS) {
	    
	    // cycle detected
	    
	    // alle Knoten auf dem aktuellen Pfad ab v liegen auf dem Zyklus
	    
	    // print path
	    
	    Iterator<DirectedGraphNode> path_iter = path.iterator();
	    
	    
	    
	    List<Integer> ring = new ArrayList<Integer>();
	    
	    ring.add(v.getId()); // close cycle
	    
	    while (path_iter.hasNext()) {
		
		int pathNodeId = path_iter.next().getId();
		
		
		
		ring.add(pathNodeId);
		
		if (pathNodeId == v.getId()) {
		    
		    cycles.add(ring);
		    
		    break;
		    
		}
		
	    }
	    
	    
	    
	} else if (v.getStatus() == DirectedGraphNode.DONE) {
	    
	    DirectedGraphNode w = path.peekFirst();
	    
	    if (w.getPreOrd() < v.getPreOrd()) {
		
		////System.out.println("forward edge (" + w.getId() + "," + v.getId() + ")");
		
		// find way from v to w in cycle
		
		List<List<Integer>> forwardEdgeCycles = new ArrayList<List<Integer>>();
		
		Iterator<List<Integer>> enumCycles = cycles.iterator();
		
		while (enumCycles.hasNext()) {
		    
		    List<Integer> way = findWayInCycle(
			    
			    enumCycles.next(), w.getId(), v.getId());
		    
		    if (way != null) {
			
			
			
			// cycle erstellen
			
			// w + way
			
			List<Integer> ring = new ArrayList<Integer>();
			
			
			
			ring.add(v.getId()); // close cycle
			
			
			
			for (int k = 0; k < way.size(); k++) {
			    
			    ring.add(way.get(k));
			    
			}
			
			
			
			// add ring to cross edge cycles
			
			forwardEdgeCycles.add(ring);
			
		    }
		    
		}
		
		// add new detected cycles by this cross edge to cycles
		
		cycles.addAll(forwardEdgeCycles);
		
		
		
	    } else {
		
		////System.out.println("cross edge (" + w.getId() + "," + v.getId() + ")");
		
		DirectedGraphNode f = path.peekLast();
		
		// find way from f to v in cycle
		
		List<List<Integer>> crossEdgeCycles = new ArrayList<List<Integer>>();
		
		Iterator<List<Integer>> enumCycles = cycles.iterator();
		
		while (enumCycles.hasNext()) {
		    
		    List<Integer> way = findWayInCycle(
			    
			    enumCycles.next(), f.getId(), v.getId());
		    
		    if (way != null) {
			
			
			
			// cycle erstellen
			
			// path + way
			
			List<Integer> ring = new ArrayList<Integer>();
			
			Iterator<DirectedGraphNode> path_iter = path.iterator();
			
			
			
			ring.add(v.getId()); // close cycle
			
			while (path_iter.hasNext()) {
			    
			    int pathNodeId = path_iter.next().getId();
			    
			    ring.add(pathNodeId);
			    
			}
			
			
			
			for (int k = 1; k < way.size(); k++) {
			    
			    ring.add(way.get(k));
			    
			}
			
			
			
			// add ring to cross edge cycles
			
			crossEdgeCycles.add(ring);
			
		    }
		    
		}
		
		// add new detected cycles by this cross edge to cycles
		
		cycles.addAll(crossEdgeCycles);
		
	    }
	    
	} else if (v.getStatus() == DirectedGraphNode.NOT_STARTED_YET) {
	    
	    v.setStatus(DirectedGraphNode.IN_PROGRESS);
	    
	    v.setPreOrd(DirectedGraph.numberPreOrd++);
	    
	    // Verl�ngere den aktuellen Suchpfad um v;
	    
	    path.addFirst(v); // push v to path stack
	    
	    Iterator<DirectedGraphNode> enumNodes = v.getAdjacentNodes().iterator();
	    
	    
	    
	    while (enumNodes.hasNext()) {
		
		
		
		DirectedGraphNode k = enumNodes.next();
		
		if (!directed) {
		    
		    k.getAdjacentNodes().remove(v);
		    
		}
		
		findCycles(k, path, directed);
		
		
		
		
		
		
		
	    }
	    
	    v.setStatus(DirectedGraphNode.DONE);
	    
	    // Entferne x (das letzte Element) aus dem aktuellen Pfad;
	    
	    path.removeFirst(); // pop v from path stack
	    
	}
	
	
	
    }
    
    
    
    /*

     * 

     */
    
    private List<Integer> findWayInCycle(List<Integer> cycle, int start,
	    
	    int end) {
	
	
	
	List<Integer> path = null;
	
	int startIndex = -1;
	
	int endIndex = -1;
	
	int i = 0;
	
	// search start index
	
	while (startIndex == -1 && i < cycle.size()) {
	    
	    if (cycle.get(i).intValue() == start) {
		
		startIndex = i;
		
	    }
	    
	    i++;
	    
	}
	
	
	
	// cycle may contain path
	
	if (startIndex > -1) {
	    
	    
	    
	    //
	    
	    i = startIndex;
	    
	    int count = 0;
	    
	    while (endIndex == -1 && count < cycle.size()) {
		
		if (cycle.get(i % cycle.size()).intValue() == end) {
		    
		    endIndex = i % cycle.size();
		    
		}
		
		i++;
		
		count++;
		
	    }
	    
	    
	    
	    if (endIndex > -1) {
		
		path = new ArrayList<Integer>();
		
		int k = startIndex;
		
		while (k % cycle.size() != endIndex) {
		    
		    path.add(cycle.get(k % cycle.size()));
		    
		    k++;
		    
		    // first and last point are identical in the cycle.
		    
		    // ignore first point when coming from last point
		    
		    // 0 can never be endIndex! Would always be last index
		    
		    // in cycle
		    
		    if (k == cycle.size()) {
			
			k++;
			
		    }
		    
		}
		
		path.add(cycle.get(endIndex));
		
	    }
	    
	}
	
	return path;
	
    }
    
    
    
    /**

     * @return the cycles

     */
    
    public List<List<Integer>> getCycles() {
	
	return cycles;
	
    }
    
    
    
    /**

     * @param cycles

     *            the cycles to set

     */
    
    public void setCycles(List<List<Integer>> cycles) {
	
	this.cycles = cycles;
	
    }
    
    /**
     * Finds the connected nodes inside a graph using the first breadth search algorithm (iterative version)
     * @return An arraylist of arraylists (an arraylist of subgraphs)
     */
    public ArrayList<ArrayList<Integer>> findConnectedNodes() {
	
	// an arraylist with all the paths (this is the return object)
	ArrayList<ArrayList<Integer>> components = new ArrayList<ArrayList<Integer>>(); 
	
	// an arraylist that holds the current path
	ArrayList<Integer> subgraph = new ArrayList<Integer>();
	
	// enumeration of the nodes
	Iterator<DirectedGraphNode> enum_nodes = nodes.values().iterator();
	
	// a stack that is going to hold all the connected polygons in a brach  
	ArrayDeque<Integer> stack = new ArrayDeque<Integer>();
	
	// poll the first polygon node
	DirectedGraphNode n1 = enum_nodes.next();
	
	// push the first polygon inside the stack
	stack.push(n1.getId());
	
	// add the first polygon id inside the subgraph
	subgraph.add(n1.getId());
	
	// mark first Node as "in progress"
	n1.setStatus(DirectedGraphNode.IN_PROGRESS);
	
	while (enum_nodes.hasNext()) {
	    
	    while (!stack.isEmpty()) {
		// get the top polygon inside the stack (it is still inside the stack)
		n1 = nodes.get(stack.getFirst());
		if (n1.getStatus() == DirectedGraphNode.DONE) {
		    stack.clear();
		    break;					
		}
		Iterator<DirectedGraphNode> iter = n1.getAdjacentNodes().iterator();
		while (iter.hasNext()) {
		    // get next polygon node and mark it
		    DirectedGraphNode ni = iter.next();
		    if (ni.getStatus() == DirectedGraphNode.IN_PROGRESS) {continue;}
		    // found a coplanar polygon so we add its id to the current subgraph
		    subgraph.add(ni.getId());
		    // add the node id to the end of the stack
		    stack.addLast(ni.getId());
		    // mark the node as "in progress"
		    ni.setStatus(DirectedGraphNode.IN_PROGRESS);
		}
		
		stack.pop();					
		
		
	    } // end while: a subgraph is finished
	    
	    // mark nodes inside the subgraph as "done"
	    for (int i : subgraph) {
		nodes.get(i).setStatus(DirectedGraphNode.DONE);
	    }
	    
	    if (subgraph.size()>1) {
		components.add(subgraph);
	    }		
	    
	    n1 = enum_nodes.next();
	    stack.push(n1.getId());
	    subgraph = new ArrayList<Integer>();
	    subgraph.add(n1.getId());
	    if (n1.getStatus() == DirectedGraphNode.NOT_STARTED_YET) {
		n1.setStatus(DirectedGraphNode.IN_PROGRESS);
	    }
	    
	} // end while
	
	return components;
    }
    
    /**
     * Returns the number of elements in this graph
     * @return
     */
    public int size() {
	return nodes.size();
    }
    
}