/** -------------------------------------------------- 
 * Hochschule fr Technik Stuttgart 
 * Fachbereich Vermessung , Informatik und Mathematik 
 * Schellingstr . 24 
 * D - 70174 Stuttgart 
 * 
 * Projekt CityDoktor 
 * 
 * Copyright (c) 2011 HFT Stuttgart. All rights reserved. 
 * HFT Stuttgart and its licensors retain all intellectual property and 
 * proprietary rights in and to this software and related documentation. 
 * Any use, reproduction, disclosure, or distribution of this software 
 * and related documentation without an express license agreement from 
 * HFT Stuttgart is strictly prohibited. 
 * 
 * Please refer to the applicable HFT Stuttgart end user license agreement (EULA) 
 * associated with this source code for terms and conditions that govern 
 * your use of this HFT Stuttgart software. 
 * 
 * @author Coors
 * @version 1.0
 * created: 26.3.2013
 */
package de.hft.stuttgart.graphtheory;

import java.util.ArrayList;
import java.util.List;




/**
 * The class edge node represents a node in a graph. The graph is stored as an adjacent list.
 * 
 * @author Coors
 *
 */
public class DirectedGraphNode {

	/*
	 * The possible status of the node while searching for cycles
	 */
	public final static int NOT_STARTED_YET=0;
	public final static int IN_PROGRESS=1;
	public final static int DONE=2;
	
	/*
	 * the id of the node
	 */
	int id;
	
	/*
	 * The status of the node while searching for cycles
	 */
	int status;
	
	/*
	 * adjacent nodes of this node in the graph.
	 */
	List<DirectedGraphNode> adjacentNodes=null;
	
	/*
	 * The number of this node during preorder traversal of the spanning tree that is created during depth search first
	 */
	int preord=0;
	
	/*
	 * construct a new node in the graph with the given id. The id has to be unique.
	 * 
	 * @param v unique node id
	 */
	public DirectedGraphNode(int v){
		id = v;
		status = NOT_STARTED_YET;
		adjacentNodes = new ArrayList<DirectedGraphNode> ();
	}
	
	/*
	 * get adjacent nodes
	 * 
	 * @return Vector<DirectedGraphNode> the adjacent nodes
	 */
	public List<DirectedGraphNode> getAdjacentNodes() {
		// TODO Auto-generated method stub
		return adjacentNodes;
	}
	
	/*
	 * add an edge from this node to node n. If the edge already exists, it will not be inserted (set of edges)
	 * 
	 * @param n the adjacent node (edge from this node to n)
	 * 
	 */
	public void addAdjacentNode(DirectedGraphNode n){
		if (!adjacentNodes.contains(n)){
			adjacentNodes.add(n);
		}
	}
	
	/*
	 * get the status of the node while searching for cycles
	 * 
	 * @return status 
	 */
	public int getStatus() {
		return status;
	}
	
	/*
	 * set the status of the node while searching for cycles
	 * 
	 * @param status 
	 */
	public void setStatus(int s){
		status=s;
	}
	
	/*
	 * get the number of this node during preorder traversal of the spanning tree that is created during depth search first
	 * 
	 * @return number of the node during preorder traversal
	 */
	public int getPreOrd() {
		return preord;
	}
	
	/*
	 * set the number of this node during preorder traversal of the spanning tree that is created during depth search first
	 * 
	 * @param number of the node during preorder traversal
	 */
	public void setPreOrd(int i){
		preord=i;
	}

	/*
	 * get the id of this node
	 * 
	 * @return node id
	 */
	public int getId() {
		return id;
	}

}
