/** -------------------------------------------------- 
* Hochschule f�r Technik Stuttgart 
* Fachbereich Vermessung , Informatik und Mathematik 
* Schellingstr . 24 
* D - 70174 Stuttgart 
* 
* Projekt CityDoktor 
* 
* Copyright (c) 2011 HFT Stuttgart. All rights reserved. 
* HFT Stuttgart and its licensors retain all intellectual property and 
* proprietary rights in and to this software and related documentation. 
* Any use, reproduction, disclosure, or distribution of this software 
* and related documentation without an express license agreement from 
* HFT Stuttgart is strictly prohibited. 
* 
* Please refer to the applicable HFT Stuttgart end user license agreement (EULA) 
* associated with this source code for terms and conditions that govern 
* your use of this HFT Stuttgart software. 
* 
* @author alam
* @version 1.0
* created: 13.09.2011
*/
package de.hft.stuttgart.graphtheory;


import java.util.ArrayList;
import java.util.List;

import de.hft.stuttgart.graphtheory.GraphNode.GRAPHSTATUS;
import de.hft.stuttgart.linearalgebra.MathUtils;

public class Graph {
	/**
	 * Constructor
	 */
	public Graph(){
		adjacentList = new ArrayList<GraphNode>();
		incedenceList = new ArrayList<GraphEdge>();
	}
	private MathUtils utils = new MathUtils();
	/**
	 * Graph Vertex or node.
	 * This is completely independent of any data type.
	 * It contains a list of nodes.
	 * A node or vertex can be anything like vertices, polygon, 
	 * edges or even e.g. people, building etc.
	 *
	 */
	private List<GraphNode> adjacentList;
	/**
	 * This contains the list of edges in the graph.
	 * Graph Edges or lines to connect graph_Vertices.
	 * Completely different from Edge class here
	 * or even any other edge class.
	 * It just connects two graph nodes.
	 */
	private List<GraphEdge> incedenceList;
	/**
	 * Getter of the property <tt>adjacentList</tt>
	 * @return the adjacentList
	 * @uml.property name = "adjacentList"
	 */
	public List<GraphNode> getAdjacentList() {
		return adjacentList;
	}
	/**
	 * Setter of the property <tt>adjacentList</tt>
	 * @param adjacentList the adjacentList to set
	 * @uml.property name = "adjacentList"
	 */
	public void setAdjacentList(List<GraphNode> adjacentList) {
		this.adjacentList = adjacentList;
	}
	/**
	 * Getter of the property <tt>incedenceList</tt>
	 * @return the incedenceList
	 * @uml.property name = "incedenceList"
	 */
	public List<GraphEdge> getIncedenceList() {
		return incedenceList;
	}
	/**
	 * Setter of the property <tt>incedenceList</tt>
	 * @param incedenceList the incedenceList to set
	 * @uml.property name = "incedenceList"
	 */
	public void setIncedenceList(List<GraphEdge> incedenceList) {
		this.incedenceList = incedenceList;
	}
	/**
	 * Finds out the edges connected to the selected nodes.
	 * @param listOfSelectedNodes
	 * @return list of GraphEdge.
	 */
	public List<GraphEdge> getListOfEdgesForSelectedNodes(List<GraphNode> listOfSelectedNodes){
		List<GraphEdge> listOfEdges = new ArrayList<GraphEdge>();
		List<Object> nodes = new ArrayList<Object>();
		for(int i=0;i<listOfSelectedNodes.size();i++){
			nodes.add(listOfSelectedNodes.get(i));
		}
		List<Object[]> combinations = utils.getCombination(nodes,2);
		for(int i=0;i<incedenceList.size();i++){
			for(int j=0;j<combinations.size();j++){
				if(incedenceList.get(i).checkIfBothNodesExist((GraphNode)combinations.get(j)[0],(GraphNode)combinations.get(j)[1]))listOfEdges.add(incedenceList.get(i));
			}
		}
		return listOfEdges;
	}
	
	public boolean checkContinuityOfGraph(){
	
		List<GraphNode> queue = new ArrayList<GraphNode>();
		adjacentList.get(0).setMarked(true);
		queue.add(adjacentList.get(0));
		
		boolean run = true;
		while(!queue.isEmpty()||!run ){
			
			AA:
				for(int i=0;i<queue.size();i++){
					for(int j=0;j<queue.get(i).getAdjucentNodes().size();j++){
							
						if(queue.get(i).getAdjucentNodes().get(j).getStatus().equals(GRAPHSTATUS.NOT_STARTED_YET)){
							
							int x = searchNodeInAdjList(queue.get(i).getAdjucentNodes().get(j));
							queue.get(i).getAdjucentNodes().get(j).setStatus(GRAPHSTATUS.IN_PROGRESS);
							adjacentList.get(x).setMarked(true);
							queue.add(adjacentList.get(x));
							break AA;
						}
					}	
					queue.remove(i);
					break AA;
				}
		}
		if (searchListForUnmarkedNode()) return false;
		else return true;
	}
	
	private int searchNodeInAdjList (GraphNode e){
		int g = -1;
		for(int i=0;i<adjacentList.size();i++){
			int a = (Integer)adjacentList.get(i).getNode();
			int b = (Integer)e.getNode();
			if(a==b){
				g=i;
				break;
			}
			
		}
		return g;
		
	}
	private boolean searchListForUnmarkedNode(){
		boolean result = false;
		for(int i=0;i<adjacentList.size();i++){
			if(!adjacentList.get(i).isMarked()) result = true;
		}
		return result;
	}
	public void createIncedentListFromAdjacentList() {
		GraphEdge gedge = new GraphEdge();
		gedge.setNode1(adjacentList.get(0));
		gedge.setNode2(adjacentList.get(0).getAdjucentNodes().get(0));
		incedenceList.add(gedge);
		
		for(int i=0;i<adjacentList.size();i++){
			GraphNode nodeA = adjacentList.get(i);
			for(int j=0;j<nodeA.getAdjucentNodes().size();j++){
				GraphNode nodeB = nodeA.getAdjucentNodes().get(j);
				gedge = new GraphEdge();
				gedge.setNode1(nodeA);
				gedge.setNode2(nodeB);
				boolean duplication = false;
				for(int k = 0; k<incedenceList.size();k++){
					if(gedge.CheckIfEqual(incedenceList.get(k))){
						duplication = true;
					}
				}
				if(!duplication)incedenceList.add(gedge);
			}
		}
		
	}
	
}
