/** -------------------------------------------------- 
 * Hochschule f?r Technik Stuttgart 
 * Fachbereich Vermessung , Informatik und Mathematik 
 * Schellingstr . 24 
 * D - 70174 Stuttgart 
 * 
 * Projekt CityDoktor 
 * 
 * Copyright (c) 2011 HFT Stuttgart. All rights reserved. 
 * HFT Stuttgart and its licensors retain all intellectual property and 
 * proprietary rights in and to this software and related documentation. 
 * Any use, reproduction, disclosure, or distribution of this software 
 * and related documentation without an express license agreement from 
 * HFT Stuttgart is strictly prohibited. 
 * 
 * Please refer to the applicable HFT Stuttgart end user license agreement (EULA) 
 * associated with this source code for terms and conditions that govern 
 * your use of this HFT Stuttgart software. 
 * 
 * @author 
 * @version 
 * 
 */


package de.hft.stuttgart.linearalgebra;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * Diese Klasse dient als Utilklasse f?r das Projekt.
 * 
 * @author Jochen Kraushaar, Stephan Gaenss, Nazmul Alam
 */
public class MathUtils {

	public double[] getDirectionVector(double[] a, double[] b){
//	public double[] getDirectionVector(Vertex a, Vertex b){

		double[] result = new double[3];
		result[0] = b[0]-a[0];
		result[1] = b[1]-a[1];
		result[2] = b[2]-a[2];
		return result;
	}

	/**
	 * Generates a combination of elements from a collection
	 * @param value The vector of any object which is the source
	 * @param r number of element in each combination
	 * @return <b>Vector of Object[]</b> where Object[] holds the elements of one combination and the size of array is same as <b>r</b>.
	 */
	public static List <Object[]> getCombination(List <?> value,int r){
		int[] indices;
		CombinationGenerator x = new CombinationGenerator (value.size(), r);
		List <Object[]> result = new ArrayList<Object[]>();
		while (x.hasMore ()) {
			Object[] instance = new Object[r];
			indices = x.getNext ();
			for (int i = 0; i < indices.length; i++) {
				instance[i]=value.get(indices[i]);
			}
			result.add(instance);
		}
		return result;
	}


	/**
	 * Checks if two polygons have the same aspect (same normalized normals)
	 * @param p1 is the first polygon
	 * @param p2 is the second polygon
	 * @param g is the geometry
	 * @param threshold in case the polygons have almost the same normalized normals.
	 * If we need exactly the same normals we set the value to 0 (zero)
	 * @return True or False
	 */
//	public static boolean sameNormalizedNormals(Polygon p1, Polygon p2, Geometry g, double threshold) {
	public static boolean sameNormalizedNormals(List<double[]> p1, List<double[]> p2, double threshold) {

		// the result
		Boolean result = false;

		// get normals
		double[] n1 = getNormalOfSurface(p1);
		double[] n2 = getNormalOfSurface(p2);

		// normalize them
		double length1 = Math.sqrt(Math.pow(n1[0], 2) + Math.pow(n1[1], 2) + Math.pow(n1[2], 2));
		double length2 = Math.sqrt(Math.pow(n2[0], 2) + Math.pow(n2[1], 2) + Math.pow(n2[2], 2));
		n1[0] = n1[0] / length1;
		n1[1] = n1[1] / length1;
		n1[2] = n1[2] / length1;

		n2[0] = n2[0] / length2;
		n2[1] = n2[1] / length2;
		n2[2] = n2[2] / length2;

		// scalar product of n1 and n2
		double n1n2 = n1[0] * n2[0] + n1[1] * n2[1] + n1[2] * n2[2];

		// if product is almost 1, polygons have same aspect
		if (n1n2 >= 1.0 - threshold) {result = true;}

		return result;
	}



	/**
	 * Calculates the perpendicular distance from the first polygon p1 to the first vertex
	 * of the second polygon p2
	 * @param p1 is the first polygon
	 * @param p2 is the second polygon
	 * @param g is the geometry object
	 * @return the perpendicular distance
	 */
//	public static double distToPlane(List<double[]> p1, List<double[]> p2, Geometry g) {
	public static double distToPlane(List<double[]> p1, List<double[]> p2) {
		// calculate the plane equation of the first polygon (Ax+By+Cz+D=0)
		double[] n1 = getNormalOfSurface(p1);
		double A = n1[0];
		double B = n1[1];
		double C = n1[2];
		double[] vertex = p1.get(0);
		double D = A * vertex[0] +
				B * vertex[1] +
				C * vertex[2];

		// get the first vertex of the second polygon
		// maybe an average coordinate pair of the plane should be calculated
		vertex = p2.get(0);

		// calculate the perpendicular distance
		double dist = (A * vertex[0] +
				B * vertex[1] +
				C * vertex[2] - D) /
				(Math.sqrt(A*A+B*B+C*C));

		return Math.abs(dist);
	}

	/**
	 * get normal of a polygon
	 * @param polygon citydoctor Polygon
	 * @return normal double[3]
	 */
	public static double[] getNormalOfSurface1(List<double[]> polygon){
		
		double nx, ny, nz;
		double[] normal = new double[3];
		double[] unitnormal = new double[3];
		
		
		//returns a null vector for polygons with 2 points or less
		if ((polygon.size() == 3 && polygon.get(0)==polygon.get(2))||(polygon.size() < 3)){
			return new double[]{0.0,0.0,0.0};
		}
				
		// iterate over all points of the polygon
		
		for (int i=0;i<polygon.size()-1;i++) {
			double[] v0 =  polygon.get(i);
			double[] v1 = polygon.get(i+1);
			nx = (v0[1]-v1[1]) * (v0[2]+ v1[2]); 
			ny = (v0[2]-v1[2]) * (v0[0]+ v1[0]); 
			nz = (v0[0]-v1[0]) * (v0[1]+ v1[1]); 
                                       
			normal[0] += nx;
			normal[1] += ny;
			normal[2] += nz;
		}
		//normalization of the normal vector
		double length = Matrix.getLengthOfVector(normal);
		unitnormal[0] = normal[0]/length;
		unitnormal[1] = normal[1]/length;
		unitnormal[2] = normal[2]/length;
		return unitnormal;
	}
	/**
	 * get normal of a polygon
	 * @param polygon citydoctor Polygon
	 * @return normal double[3]
	 */
	public static double[] getNormalOfSurface(List<double[]> polygon){
		
		double nx, ny, nz;
		double[] normal = new double[3];
		double[] unitnormal = new double[3];
		
		
		//returns a null vector for polygons with 2 points or less
		if ((polygon.size() == 3 && polygon.get(0)==polygon.get(2))||(polygon.size() < 3)){
			return new double[]{0.0,0.0,0.0};
		}
				
		// iterate over all points of the polygon
		
		for (int i=0;i<polygon.size()-1;i++) {
			double[] v0 =  polygon.get(i);
			double[] v1 = polygon.get(i+1);
			nx = (v0[1]-v1[1]) * (v0[2] + v1[2]); 
			ny = (v0[2]-v1[2]) * (v0[0] + v1[0]); 
			nz = (v0[0]-v1[0]) * (v0[1] + v1[1]); 

			normal[0] += nx;
			normal[1] += ny;
			normal[2] += nz;
		}
		//normalization of the normal vector
		double length = Matrix.getLengthOfVector(normal);
		unitnormal[0] = normal[0]/length;
		unitnormal[1] = normal[1]/length;
		unitnormal[2] = normal[2]/length;
		return unitnormal;
	}

	/**
	 * Checks if the coordinates are null
	 * @param coord coordinates of a vertex
	 * @throws Exception
	 */
	protected void checkCoordinate(double[] coord) throws Exception{
		if(coord.length!=3)throw new Exception("Invalid length of array for Coordinate");
		// TODO MW: was sollte hier geschehen? F�r mich ergeben diese Pr�fungen keinen Sinn. 
//		if(Objects.isNull(coord[0]))throw new Exception("X Coordinate is Null");
//		if(Objects.isNull(coord[1]))throw new Exception("Y Coordinate is Null");
//		if(Objects.isNull(coord[2]))throw new Exception("Z Coordinate is Null");
	}
	/**
	 * Checks if the citydoctor LinearRing is valid
	 * @param linearRing citydoctor LinearRing
	 * @throws Exception
	 */
	protected void CheckLinearRing(List<double[]> linearRing) throws Exception{
		if(linearRing.size()<3)throw new Exception("Invalid length of array for Coordinate");
		for(double[] coord:linearRing){
			checkCoordinate(coord);
		}
	}
	/**
	 * Calculate length of a line
	 * @param a 1st point
	 * @param b 2nd point
	 * @return length
	 */
	public static double getLengthOfLine(double[] a, double[] b){
		if(a==null||b==null)return -1;
		double length=Math.sqrt(	(b[0]-a[0])*(b[0]-a[0])+
									(b[1]-a[1])*(b[1]-a[1])+
									(b[2]-a[2])*(b[2]-a[2]));
		return length;
	}
	public static String toString(double[] array){
		NumberFormat nf = NumberFormat.getInstance();

		nf.setMaximumFractionDigits(4);
		StringBuffer buffer = new StringBuffer();
		if(array.length>0){
			
			for(double x:array){
				buffer.append(nf.format(x));
				buffer.append('-');
			}
			buffer.delete(buffer.length()-1, buffer.length());
			
		}
		
		return buffer.toString();
	}
}
