/** -------------------------------------------------- 
* Hochschule f�r Technik Stuttgart 
* Fachbereich Vermessung , Informatik und Mathematik 
* Schellingstr . 24 
* D - 70174 Stuttgart 
* 
* Projekt CityDoktor 
* 
* Copyright (c) 2011 HFT Stuttgart. All rights reserved. 
* HFT Stuttgart and its licensors retain all intellectual property and 
* proprietary rights in and to this software and related documentation. 
* Any use, reproduction, disclosure, or distribution of this software 
* and related documentation without an express license agreement from 
* HFT Stuttgart is strictly prohibited. 
* 
* Please refer to the applicable HFT Stuttgart end user license agreement (EULA) 
* associated with this source code for terms and conditions that govern 
* your use of this HFT Stuttgart software. 
* 
* @author alam
* @version 
* 
*/ 
package de.hft.stuttgart.linearalgebra;

/**
 * @author alam
 *
 */
public class Matrix {
	/**
	 * This function performs the Scalar Multiplication between two vectors
	 * @param a the first vector
	 * @param b the second vector
	 * @return result of Scalar Multiplication between two vectors
	 */
	public static double getVectorScalarMultiplication(double[] a, double[] b){
		double result =0.0;
		if(Compare.ifEqual(a.length,b.length)){
			for(int i=0;i<a.length;i++){
				result=result+a[i]*b[i];
			}
		}
		return result;
	}
	/**
	 * This function performs the Cross Multiplication between two vectors
	 * @param a the first vector
	 * @param b the second vector
	 * @return the cross-product a.k.a the normal vector...if 0|0|0 the two
	 *         vectors are linear dependent or one of them is 0.
	 */
	public static double[] getVectorCrossMultiplication(double[] a, double[] b){
		double[] result = new double[3];
		result[0] = (a[1] * b[2]) - (a[2] * b[1]);
		result[1] = (a[2] * b[0]) - (a[0] * b[2]);
		result[2] = (a[0] * b[1]) - (a[1] * b[0]);
		return result;
	}
	/**
	 * This function calculates the angle between two vectors
	 * 
	 * @param vec1
	 *            the first vector
	 * @param vec2
	 *            the second vector
	 * @return the inner angle the vectors form (in degrees)
	     */
	public double getAngleBetweenTwoVector(double[] a, double[] b){
		double result = (180/Math.PI)*Math.acos(Matrix.getVectorScalarMultiplication(a, b)/(getLengthOfVector(a)*getLengthOfVector(b)));

		return result;
	}

	/**
	 * Returns the norm of a vector
	 * @param a double[] the vector
	 * @return result double the norm of the vector
	 */	
	public static double getLengthOfVector(double[] a){
		double result=0.0;
		for(int i=0;i<a.length;i++){
			result=result+a[i]*a[i];
		}
		result = Math.sqrt(result);
		return result;
	}
	/**
	 * normalizes a vector in 3D space
	 * @param v double[] the vector
	 * @return result double the normalized vector
	 */
	public static double[] normalize(double[] v) {
        double length = getLengthOfVector(v);
        v[0] = v[0] / length;
        v[1] = v[1] / length;
        v[2] = v[2] / length;
        return v;
    }


	
	/**
	 * 0° North, 90° East, 180° South and 270° West
	 * 
	 * @param n
	 * @return azimuth
	 */
	public static double calcAzimuth(double[] n) {
		double epsilon = 0.0001;
		double az;
		if (Math.abs(n[2]) > 1 - epsilon) {
			// NOTE: By convention. Flat roofs and grounds are oriented to the south
			az = 180;
		} else {
			az = ((-(Math.atan2(n[1], n[0]) / Math.PI * 180) + 90) % 360 + 360) % 360;
		}
		return az;
	}

	/**
	 * 0° is flat, 90° is facade, 180° is ground
	 * 
	 * @param n
	 * @return tilt
	 */
	public static double calcTilt(double[] n) {
		
		return Math.acos(n[2]/(Math.sqrt(n[0]*n[0]+n[1]*n[1]+n[2]*n[2]))) * 180 / Math.PI;
	}
}
