package de.hft.stuttgart.volumecalculator;



import java.util.List;

/**
 * You may call this calculator directly if you want, but we encourage you to use an appropriate 
 * feature handler (BuildingHandler, GeometryHandler) instead. The feature handlers encapsulate
 * volume calculators for you and provide a more concise and object-oriented scheme.
 * 
 * @author alam
 */
public class VolumeCalculator {
		
	private static double[] v;
	


	
	/**
	 * Calculates volume of a geometry.
	 * 
	 * @param geometry The geometry for which the volume has to be calculated.
	 * @return volume The volume of the geometry.
	 */
	public static double calculateVolume(List<List<double[]>> geometry) throws VolumeCalculationException {
		double sum = 0;
		for(List<double[]> t : geometry) {
			for (int i=0; i<3; i++) {
				if (v == null) {
					v = t.get(i);
				} else if (v[0]>t.get(i)[0]) {
					v = t.get(i);
				}
			}
		}
		for (List<double[]> t : geometry) {
			if (t.size()!=3) {
				throw new VolumeCalculationException();
			}
			sum += det(t.get(0), t.get(1), t.get(2));
		}
		return (sum/6);
	}
	
	/** Calculate the determinant of a 3x3-matrix. */
	private static double det(double[] a, double[] b, double[] c) {
		double det = - c[0] * b[1] * a[2]
				 + b[0] * c[1] * a[2]
				 + c[0] * a[1] * b[2]
				 - a[0] * c[1] * b[2]
				 - b[0] * a[1] * c[2]
				 + a[0] * b[1] * c[2];
		return det;
	}
	


}